#!/bin/sh
# Dependencies: synfig, imagemagick

# Copyright (C) 2010-2020 Duncan Deveaux
# 
# This file is part of Hikou no mizu.
# 
# Hikou no mizu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Hikou no mizu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.


# animbuild.sh: Builds a spritesheet png file from a sifz animation.

base_srcdir="$1"    # datasrc dir
base_outputdir="$2" # generated data dir
output_name="$3"    # relative name of output spritesheet (e.g. Hikou/default.png)

workingdir="$4" # Temporary working directory

imgwidth="$5"  # Width of the generated image
imgheight="$6" # Height of the generated image
tiles="$7"     # Number of sprites per row

inputs="$8" # Input sifz animations description


pngdest="$base_outputdir$output_name" # Output png spritesheet

mkdir -p "$workingdir"
mkdir -p $(dirname $pngdest)

# Render animation frames
i=1
input_i=$(printf '%s;' "$inputs" | cut -d\; -f1)
input_i=$(printf '%s' "$input_i" | tr -d ' \n[]}') #Trim
while ! [ -z "$input_i" ] ; do

    # Browse input sifz descriptions
    printf '%s\n' "$input_i" |
    while IFS='-' read -r sifz_file start end ; do
        mkdir "$workingdir/tmp$i"
        synfig "$base_srcdir$sifz_file" -t png -Q 10 -o "$workingdir/tmp$i/render.png" > /dev/null 2>&1
            
        # Register the needed frames
        if [ "$end" = ":" ] ; then
            # Select from 'start' to the last frame
            added_frames=$(ls $workingdir/tmp$i/* | sed -n "${start}~1p")
        else
            # Select from 'start' to 'end'
            added_frames=$(ls $workingdir/tmp$i/* | sed -n "${start},${end}p")
        fi
        
        printf '%s\n' "$added_frames" >> "$workingdir/frames"
    done
    
    # Next input
    i=$(($i+1))
    input_i=$(printf '%s;' "$inputs" | cut -d\; -f$i)
    input_i=$(printf '%s' "$input_i" | tr -d ' \n[]}') #Trim
done

# Print needed frames
input_frames=$(cat "$workingdir/frames")
montage $input_frames -background none -geometry +2+2 -tile ${tiles}x "$workingdir/tiles.png"

# Resize canvas
convert "$workingdir/tiles.png" -background none -gravity NorthWest -chop 2x2+0+0 -extent ${imgwidth}x${imgheight} "$pngdest"

printf '  %-48sOK\n' "$output_name"
rm -rf "$workingdir"
