/*Copyright (C) 2010-2020 Duncan Deveaux

This file is part of Hikou no mizu.

Hikou no mizu is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Hikou no mizu is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Hikou no mizu.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEF_PLATFORM
#define DEF_PLATFORM

#include "Graphics/Drawable/Sprite.hpp"
#include "Engines/Physics/PhysicsObject.hpp"
#include <iostream>

//PhysicsObject's inheriting class representing a simple textured platform.
class Platform : public PhysicsObject
{
    public:
        Platform();
        void draw();
        
        //Get and set methods
        void setBodyBox(const Box &bodyBox);
        const Box &getBodyBox() const;
        
        void setSprite(const Sprite &sprite);
        const Sprite &getSprite() const;
        
        void setGraphicsSize(float width, float height);
        
        void collideWorld();
        void collide(PhysicsObject& /*obj*/);
        void takeAHit(float strength, int side, const PhysicsObject* /*source*/);
        
        void setHitable(bool hitable);
        bool isHitable() const;
        
        bool interceptsShadows() const;
    
    private:
        void resizeSprite();
        
        Box m_bodyBox;
        Sprite m_sprite;
        float m_graphicsWidth, m_graphicsHeight;
        
        bool m_hitable;
};

#endif
